module Wire

push!(LOAD_PATH, pwd())
using Meshes
using GfxBase
using LinearAlgebra

using Luxor # a 2D drawing library

function cube_to_tris()
    cube = cube_mesh()
    triangles = []
    for tri in cube.triangles
        vertices = [] 
        for v = 1:3
            push!(vertices, Vec4(cube.positions[tri.positions[v]]..., 1.0))
        end
        push!(triangles, vertices)
    end
    triangles
end

function main()

    # Viewport specs
    cw = ch = 400 # canvas dimensions

    # Task 2: make the cube 40x40
    # Task 3: Rotate it 30 degrees
    # around the y axis and move it to be centered 
    # at (0, 1, -4).

    R = [cosd(30) 0 -sind(30) 0;
         0        1  0        0;
         sind(30) 0  cosd(30) 0;
         0        0  0        1]

    model =[20 0 0 0;
            0 20 0 30;
            0 0 20 -4;
            0 0 0 1]*R
    # Task 4: Move the camera 20 units in the +y
    # direction.
    cam  = [1 0 0 0;
            0 1 0 -20;
            0 0 1 0;
            0 0 0 1]            

    proj = [1 0 0 0;
            0 1 0 0;
            0 0 1 0;
            0 0 0 1]

    # Task 1: put the world origin in the middle 
    # of pixel space
    vp   = [1 0 0 cw/2;
            0 1 0 ch/2;
            0 0 1 0;
            0 0 0 1]



    M = vp*proj*cam*model
    Drawing(cw, ch)
    background("white")
    setline(0.5)

    for tri in cube_to_tris()
        # transform each vertex
        v1 = M * tri[1]
        v2 = M * tri[2]
        v3 = M * tri[3]
        v1 = Point(v1[1], v1[2]) # drop z and w!
        v2 = Point(v2[1], v2[2])
        v3 = Point(v3[1], v3[2])

        # nb: Luxor's default is to have (0,0) at the top
        # left with x going right and y going down
        line(v1, v2, :stroke)
        line(v2, v3, :stroke)
        line(v3, v1, :stroke)

    end
    finish()
    preview()
end


end # module Wire

Wire.main()